//
//  ES1Renderer.m
//  hw2_graphics
//
//  Created by slork on 1/12/10.
//  Copyright Stanford University 2010. All rights reserved.
//

#import "ES1Renderer.h"
#import "mopho.h"
#import "GLUtils.h"
#import "GLObject.h"

// Ocarina Layout:
//
//            HOLE3
//    HOLE1
//
//            HOLE2
//    HOLE0
//

// GLObjects id
#define HOLE0     0
#define HOLE1     1
#define HOLE2     2
#define HOLE3     3

#define CIRCLE0   4
#define CIRCLE1   5
#define CIRCLE2   6
#define CIRCLE3   7

// defines
#define SRATE           16000
#define FRAMESIZE       512
#define MAXOBJECTS      12
#define NUMCHANNELS     2

// Holes Positions
#define HOLE0POS    CGPointMake(85.0, 350.0)
#define HOLE1POS    CGPointMake(85.0, 175.0)
#define HOLE2POS    CGPointMake(245.0, 285.0)
#define HOLE3POS    CGPointMake(245.0, 100.0)

// Holes Sizes
#define HOLE0SIZE   50.0
#define HOLE1SIZE   40.0
#define HOLE2SIZE   30.0
#define HOLE3SIZE   20.0

#define HOLEMARGIN  2.0

// Compass
#define NORTH (g_headingValue > 320 || g_headingValue < 40)
#define EAST (g_headingValue > 50 && g_headingValue < 140)
#define WEST (g_headingValue > 230 && g_headingValue < 310)

// Reverb
#define REV_T60 0.95

// Notes
#define G3  196.00
#define GS3	207.65
#define A3  220.00
#define AS3 233.08 
#define B3 	246.94 
#define C4 	261.63 
#define CS4 277.18 
#define D4 	293.66 
#define DS4 311.13 
#define E4 	329.63 
#define F4 	349.23 
#define FS4 369.99 
#define G4 	392.00 
#define GS4 415.30 
#define A4 	440.00 
#define AS4 466.16 
#define B4 	493.88 
#define C5 	523.25 
#define CS5 554.37 
#define D5 	587.33 
#define DS5 622.25
#define E5 	659.26 
#define F5 	698.46 
#define FS5 739.99 
#define G5 	783.99 
#define GS5 830.61 
#define A5 	880.00
#define AS5 932.33
#define B5 	987.77
#define C6 	1046.50 	

/////////////////////////
#pragma mark Globals
/////////////////////////

GLObject *g_objects[MAXOBJECTS];  // GLObjects array
GLuint g_texture[1];              // Textures
GLfloat g_accelx = 0;             // acceleration x
GLfloat g_accely = 0;             // acceleration y

// compass heading
CLLocationDirection g_headingValue;

// settings
Vector3D g_color(0.0, 0.65, 0.9); // Blue by default
bool g_modulation = true;         // Modulation On by default
bool g_reverb = true;             // Reverb On by default
bool g_glissando = false;         // Glissando Off by default
StkFloat g_glissando_coef = 1.0;  // Glissando Coefficient (1 by default)

/////////////////////////
#pragma mark Callbacks
/////////////////////////

// implement audio callback here
void audioCallback( Float32 * buffer, UInt32 frameSize, void * userData) {
  
  AudioData * data = (AudioData*)userData;
  
  // Update the Note
  updateNote(data);
  
  if (data->curr_note == 0) {
    data->curr_note = data->note;
  }
  
  // Glissando
  if (g_glissando) {
    // Increase frequency if current note is lower than the goal note
    if (data->curr_note < data->note) {
      data->curr_note += g_glissando_coef*10;
    }
    // Decrease if it's higher than the goal note
    else if (data->curr_note > data->note) {
      data->curr_note -= g_glissando_coef*10;
    }
  }
  // No glissando, assign goal note to current note
  else {
    data->curr_note = data->note;
  }
  
  // Assign Note
  if (data->note != 0) {
    data->fm->noteOn(data->curr_note, 1.0);
  }
  
  // Write to the output buffer
  for (int i=0; i<frameSize; i++) {
    if (g_reverb) {
      buffer[2*i] = buffer[2*i + 1] = 0.7*data->reverb->tick(data->fm->tick());
    }
    else {
      buffer[2*i] = buffer[2*i + 1] = 0.8*data->fm->tick();
    }

  }
  
}

// Compass callback
void compassCallback( CLHeading * heading, void * data ) {
  
  g_headingValue = [heading magneticHeading];
  
  
  //sendCompassOSC(g_headingValue);
}


// Accelerometer Callback
void accelCallback( double x, double y, double z, void * userData) {
  
  AudioData * data = (AudioData*)userData;
  
  //NSLog(@"x: %f, y: %f, z: %f, heading: %f", x,y,z, g_headingValue);
  
  if (g_modulation) {
    data->fm->setModulationDepth(fabs(y));
  }
  
}


// Touch Callback
void touchCallback( NSSet * touches, UIView *view, const std::map<int, 
                   MoTouchTrack> & touchPts, void * userData  ) {
  
  //NSLog(@"Touch");
  
  // If we are in a diferent view, do nothing
  if ([view tag] != 666) {
    return;
  }
  
  for (UITouch *touch in touches) {
    
    CGPoint point = [touch locationInView:view];
    
    if ([touch phase] == UITouchPhaseBegan) {
      
      if ( pointWithinCircle(point, HOLE0POS, HOLE0SIZE)) {
        ((GLCircle*)g_objects[CIRCLE0])->setPressed(true);
        ((GLCircle*)g_objects[CIRCLE0])->touch = touch;
      }
      
      else if ( pointWithinCircle(point, HOLE1POS, HOLE0SIZE)) {
        ((GLCircle*)g_objects[CIRCLE1])->setPressed(true);
        ((GLCircle*)g_objects[CIRCLE1])->touch = touch;
      }
      
      else if ( pointWithinCircle(point, HOLE2POS, HOLE0SIZE)) {
        ((GLCircle*)g_objects[CIRCLE2])->setPressed(true);
        ((GLCircle*)g_objects[CIRCLE2])->touch = touch;
      }
      
      else if ( pointWithinCircle(point, HOLE3POS, HOLE0SIZE)) {
        ((GLCircle*)g_objects[CIRCLE3])->setPressed(true);
        ((GLCircle*)g_objects[CIRCLE3])->touch = touch;
      }
    }
    
    else if ([touch phase] == UITouchPhaseEnded || [touch phase] == UITouchPhaseCancelled) {
      
      if ( pointWithinCircle(point, HOLE0POS, HOLE0SIZE)) {
        ((GLCircle*)g_objects[CIRCLE0])->setPressed(false);
        ((GLCircle*)g_objects[CIRCLE0])->touch = NULL;
      }
      
      else if ( pointWithinCircle(point, HOLE1POS, HOLE0SIZE)) {
        ((GLCircle*)g_objects[CIRCLE1])->setPressed(false);
        ((GLCircle*)g_objects[CIRCLE1])->touch = NULL;
      }
      
      else if ( pointWithinCircle(point, HOLE2POS, HOLE0SIZE)) {
        ((GLCircle*)g_objects[CIRCLE2])->setPressed(false);
        ((GLCircle*)g_objects[CIRCLE2])->touch = NULL;
      }
      
      else if ( pointWithinCircle(point, HOLE3POS, HOLE0SIZE)) {
        ((GLCircle*)g_objects[CIRCLE3])->setPressed(false);
        ((GLCircle*)g_objects[CIRCLE3])->touch = NULL;
      }
      
    }
    
    
    else if ([touch phase] == UITouchPhaseMoved) {
      
      bool pressed;
      
      // If we have some touches, update them
      if ( ((GLCircle*)g_objects[CIRCLE0])->touch == touch ) {
        pressed = pointWithinCircle(point, HOLE0POS, HOLE0SIZE);
        ((GLCircle*)g_objects[CIRCLE0])->setPressed(pressed);
        ((GLCircle*)g_objects[CIRCLE0])->touch = (pressed) ? touch : NULL;
      }
      
      else if ( ((GLCircle*)g_objects[CIRCLE1])->touch == touch ) {
        pressed = pointWithinCircle(point, HOLE1POS, HOLE0SIZE);
        ((GLCircle*)g_objects[CIRCLE1])->setPressed(pressed);
        ((GLCircle*)g_objects[CIRCLE1])->touch = (pressed) ? touch : NULL;
      }
      
      else if ( ((GLCircle*)g_objects[CIRCLE2])->touch == touch ) {
        pressed = pointWithinCircle(point, HOLE2POS, HOLE0SIZE);
        ((GLCircle*)g_objects[CIRCLE2])->setPressed(pressed);
        ((GLCircle*)g_objects[CIRCLE2])->touch = (pressed) ? touch : NULL;
      }
      
      else if ( ((GLCircle*)g_objects[CIRCLE3])->touch == touch ) {
        pressed = pointWithinCircle(point, HOLE3POS, HOLE0SIZE);
        ((GLCircle*)g_objects[CIRCLE3])->setPressed(pressed);
        ((GLCircle*)g_objects[CIRCLE3])->touch = (pressed) ? touch : NULL;
      }
      
      // We don't have any touch yet, we might want to add one
      else {
        if ( pointWithinCircle(point, HOLE0POS, HOLE0SIZE)) {
          ((GLCircle*)g_objects[CIRCLE0])->setPressed(true);
          ((GLCircle*)g_objects[CIRCLE0])->touch = touch;
        }
        
        else if ( pointWithinCircle(point, HOLE1POS, HOLE0SIZE)) {
          ((GLCircle*)g_objects[CIRCLE1])->setPressed(true);
          ((GLCircle*)g_objects[CIRCLE1])->touch = touch;
        }
        
        else if ( pointWithinCircle(point, HOLE2POS, HOLE0SIZE)) {
          ((GLCircle*)g_objects[CIRCLE2])->setPressed(true);
          ((GLCircle*)g_objects[CIRCLE2])->touch = touch;
        }
        
        else if ( pointWithinCircle(point, HOLE3POS, HOLE0SIZE)) {
          ((GLCircle*)g_objects[CIRCLE3])->setPressed(true);
          ((GLCircle*)g_objects[CIRCLE3])->touch = touch;
        }        
      }
      
    }
    
  }
}

/////////////////////////////
#pragma mark Custom Methods
/////////////////////////////

// Update the note to play depending on the holes pressed
void updateNote(AudioData *data) {
  
  bool hole0 = ((GLCircle*)g_objects[CIRCLE0])->isPressed();
  bool hole1 = ((GLCircle*)g_objects[CIRCLE1])->isPressed();
  bool hole2 = ((GLCircle*)g_objects[CIRCLE2])->isPressed();
  bool hole3 = ((GLCircle*)g_objects[CIRCLE3])->isPressed();
  
  if (!hole0 && !hole1 && !hole2 && !hole3) {
    data->fm->noteOff(1.0);
    data->note = 0.0;
    data->curr_note = 0.0;
  }
  
  else if (!hole0 && !hole1 && hole2 && hole3) {
    data->note = G3;
  }
  
  else if (!hole0 && hole1 && hole2 && !hole3) {
    data->note = A3;
  }
  
  else if (!hole0 && hole1 && !hole2 && hole3) {
    data->note = AS3;
  }
  
  else if (!hole0 && hole1 && hole2 && hole3) {
    data->note = B3;
  }
  
  else if (hole0 && hole1 && hole2 && hole3) {
    data->note = C4;
  }
  
  else if (hole0 && hole1 && hole2 && !hole3) {
    data->note = D4;
  }
  
  else if (hole0 && hole1 && !hole2 && hole3) {
    data->note = E4;
  }
  
  else if (hole0 && hole1 && !hole2 && !hole3) {
    data->note = F4;
  }
  
  else if (hole0 && !hole1 && hole2 && hole3) {
    data->note = FS4;
  }
  
  else if (hole0 && !hole1 && hole2 && !hole3) {
    data->note = G4;
  }
  
  else if (hole0 && !hole1 && !hole2 && hole3) {
    data->note = GS4;
  }
  
  else if (hole0 && !hole1 && !hole2 && !hole3) {
    data->note = A4;
  }
  
  else if (!hole0 && !hole1 && hole2 && !hole3) {
    data->note = AS4;
  }
  
  else if (!hole0 && !hole1 && !hole2 && hole3) {
    data->note = B4;
  }
  
  else if (!hole0 && hole1 && !hole2 && !hole3) {
    data->note = C5;
  }
  
}


// Send OSC Compass Information
void sendCompassOSC (float headingValue) {
  
  const int num_vars = 1;
  char types[num_vars] = { 'f' };
  
  MoNet::sendMessage("192.168.180.112", 8888, "/compass", types, num_vars, headingValue);
}

// Process OSC Input
void processInput (osc::ReceivedMessageArgumentStream & oscin, void* data) {
  
  const char *x;
  oscin >> x;
}

// Creates a Circle Object and adds it to the objects array
void createCircle(CGPoint point, CGFloat size, Vector3D color, 
                  CGFloat line_width, bool filled, 
                  UITouch *touch, int touch_id)  {
  
  GLCircle *circle = new GLCircle(24, point, size, color, 4.0f, filled);
  circle->touch = touch;
  
  // Add circle to objects array
  g_objects[touch_id] = circle;
  
}

// Checks whether a point is within a cercle or not
bool pointWithinCircle(CGPoint p, CGPoint center, CGFloat size) {
  
  bool res = false;
  
  if ( p.x < center.x + size && p.x > center.x - size &&
      p.y < center.y + size && p.y > center.y - size ) {
    res = true;
  }
  
  return res;
}

void initHoles() {
  
  g_objects[HOLE0] = new GLHole( HOLE0POS, HOLE0SIZE, g_color, NULL );
  g_objects[HOLE1] = new GLHole( HOLE1POS, HOLE1SIZE, g_color, NULL );
  g_objects[HOLE2] = new GLHole( HOLE2POS, HOLE2SIZE, g_color, NULL );
  g_objects[HOLE3] = new GLHole( HOLE3POS, HOLE3SIZE, g_color, NULL );
}

void initCircles() {
  
  // Create Hole 0
  createCircle(HOLE0POS, HOLE0SIZE + HOLEMARGIN, g_color, 4.0f, false, NULL, CIRCLE0);
  
  // Create Hole 1
  createCircle(HOLE1POS, HOLE1SIZE + HOLEMARGIN, g_color, 4.0f, false, NULL, CIRCLE1);
  
  // Create Hole 2
  createCircle(HOLE2POS, HOLE2SIZE + HOLEMARGIN, g_color, 4.0f, false, NULL, CIRCLE2);
  
  // Create Hole 3
  createCircle(HOLE3POS, HOLE3SIZE + HOLEMARGIN, g_color, 4.0f, false, NULL, CIRCLE3);
}


// Draw function
void draw(AudioData * data) {
  
  glPushMatrix();
  
  // For all possible objects, draw them
  for (int i = 0; i < MAXOBJECTS; i++) {
    if (g_objects[i]) {
      g_objects[i]->color = g_color;
      g_objects[i]->draw();
    }
  }
  
  glPopMatrix();
}

@implementation ES1Renderer

// Create an ES 1.1 context
- (id) init
{
	if (self = [super init])
	{
		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
        
        if (!context || ![EAGLContext setCurrentContext:context])
		{
            [self release];
            return nil;
        }
		
		// Create default framebuffer object. The backing will be allocated for the current layer in -resizeFromLayer
		glGenFramebuffersOES(1, &defaultFramebuffer);
		glGenRenderbuffersOES(1, &colorRenderbuffer);
		glBindFramebufferOES(GL_FRAMEBUFFER_OES, defaultFramebuffer);
		glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
		glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, colorRenderbuffer);
	}
  
  // enable texture mapping
  glEnable( GL_TEXTURE_2D );
  // enable blending
  glEnable( GL_BLEND );
  // blend function
  // glBlendFunc( GL_ONE, GL_ZERO );
  glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
  
  // generate texture name
  glGenTextures( 1, &g_texture[0] );
  // bind the texture
  glBindTexture( GL_TEXTURE_2D, g_texture[0] );
  // setting parameters
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  // load the texture
  MoGfx::loadTexture( @"hole", @"png" );
  
  // Setup STK itself
  Stk::setSampleRate(SRATE);
  Stk::showWarnings(false);
  
  // Setup STK Instruments
  audioData.sineWave = new SineWave();
  audioData.adsr = new ADSR();
  //audioData.reverb = new JCRev();
  audioData.reverb = new NRev();
  audioData.fm = new FMVoices();
  
  audioData.sineWave->setFrequency(440.0);
  audioData.fm->setFrequency(220.0);
  audioData.fm->noteOff(1.0);
  
  audioData.reverb->setT60(REV_T60);
  
  // Init Holes
  initHoles();
  
  // Init Circles
  initCircles();
  
  // Init Note
  audioData.curr_note = 0.0;
  audioData.note = 0.0;
  
  // Init Touch
  MoTouch::addCallback( touchCallback, &audioData );
  
  // Init Audio
  bool result = MoAudio::init( SRATE, FRAMESIZE, NUMCHANNELS );
  
  if (!result) {
    NSLog(@"cannot initialize real-time audio!");
    return NULL;
  }
  
  // start audio layer
  result = MoAudio::start( audioCallback, &audioData);
  
  if (!result) {
    NSLog(@"cannot start real-time audio!");
    return NULL;
  }  
  
  // Init Compass
  //MoCompass::addCallback(compassCallback, NULL);
  
  // Init Accel
  MoAccel::addCallback( accelCallback, &audioData );
  
  // Receiver
  //MoNet::addAddressCallback("/incoming", processInput, NULL);
  //MoNet::setListeningPort(8888);
  //MoNet::startListening();
	
	return self;
}

- (void) render
{
	
	// This application only creates a single context which is already set current at this point.
	// This call is redundant, but needed if dealing with multiple contexts.
  [EAGLContext setCurrentContext:context];
  
	// This application only creates a single default framebuffer which is already bound at this point.
	// This call is redundant, but needed if dealing with multiple framebuffers.
  glBindFramebufferOES(GL_FRAMEBUFFER_OES, defaultFramebuffer);
  glViewport(0, 0, backingWidth, backingHeight);
  
  // We desable the Depth since we are in 2D Mode
  glDisable(GL_DEPTH_TEST);
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrthof (0.0f, 320.0f, 480.0f, 0, -1, 1);
  glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
  // Background Color
  glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
  glClear(GL_COLOR_BUFFER_BIT);
  
  // Draw objects
  draw(&audioData);
  
	// This application only creates a single color renderbuffer which is already bound at this point.
	// This call is redundant, but needed if dealing with multiple renderbuffers.
  glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
  [context presentRenderbuffer:GL_RENDERBUFFER_OES];
}

- (BOOL) resizeFromLayer:(CAEAGLLayer *)layer
{	
	// Allocate color buffer backing based on the current layer size
  glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
  [context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:layer];
	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &backingWidth);
  glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &backingHeight);
	
  if (glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES)
	{
		NSLog(@"Failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
    return NO;
  }
  
  return YES;
}

- (void) dealloc
{
	// Tear down GL
	if (defaultFramebuffer)
	{
		glDeleteFramebuffersOES(1, &defaultFramebuffer);
		defaultFramebuffer = 0;
	}

	if (colorRenderbuffer)
	{
		glDeleteRenderbuffersOES(1, &colorRenderbuffer);
		colorRenderbuffer = 0;
	}
	
	// Tear down context
	if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
	
	[context release];
	context = nil;
	
	[super dealloc];
}

@end
