/*
 *  GLObject.h
 *  hw2
 *
 *  Created by Uri Nieto on 1/24/10.
 *  Copyright 2010 Stanford University. All rights reserved.
 *
 */

#import <UIKit/UIKit.h>
#import <OpenGLES/ES1/gl.h>
#import <OpenGLES/ES1/glext.h>
#import "mo_gfx.h"
#import "ES1Renderer.h"

// defines
#define CIRCLEANIMATION 20
#define CIRCLEMAXSIZE   size * 1.6
#define CIRCLEANIMATIONTIME 2.5
#define EXPLOSIONANIMATION  400
#define NOTSET          -1
#define TIMEINCREMENT   0.05
#define ACCELERATION    80.0


// GLObject super class 
// (all its subclasses *must* implement the draw method)
class GLObject {
  
public:
  GLsizei count;      // number of vertices
  CGPoint center;     // center of the object
  CGFloat size;       // size of the object
  Vector3D color;     // color of the object [r,g,b]
  GLfloat alpha;      // Alpha of the object
  GLfloat line_width; // line width of the object
  bool filled;        // the object will be filled?
  int type;           // type of the object (Circle, line, ...)
  int key;            // Key assigned by the MoPho library
  UITouch *touch;     // Actual touch of the object
  
  virtual void draw() {};
  
protected:
  CGFloat dt;         // Time increment for Physical Modeling
};

// Circle Class, can be a Touch or Launcher
class GLCircle : public GLObject {
  
public:
  GLCircle( GLsizei count, CGPoint center, CGFloat size, 
           Vector3D color, GLfloat line_width, bool filled );
  
  void draw();
  bool isPressed();
  void setPressed(bool p);
  
private:
  bool animateOn;
  CGFloat size_animating; //size of animation
  CGFloat zrot;           //zrotation
  bool pressed;
  
  CGFloat doExpandAnimation();
  CGFloat doShrinkAnimation();
  void doRotateAnimation();
};

// Explosion Class
class GLExplosion : public GLObject {
  
public:
  GLExplosion( GLsizei count, CGPoint center, CGFloat size, 
           GLubyte *color, GLfloat line_width, bool filled );
  
  void draw();
  
private:
  bool animateOn;
  int animation_time;     //animation when start
  CGFloat size_animating; //size of animation
  CGFloat zrot;           //zrotation
  
  CGFloat doSizeAnimation();
  void doRotateAnimation();
};

// Hole Class, doesn't move, no animations
class GLHole : public GLObject {
  
public:
  GLHole( CGPoint center, CGFloat size, Vector3D color, AudioData * data );
  
  void draw();
  
private:
  bool animateOn;
  CGFloat zrot;           //zrotation
  CGFloat size_animating; //size while animating
  AudioData * data;
  
  CGFloat doSizeAnimation();
};
