/*
 *  GLObject.mm
 *  hw2
 *
 *  Created by Uri Nieto on 1/24/10.
 *  Copyright 2010 Stanford University. All rights reserved.
 *
 */

#include "GLObject.h"
#include "GLUtils.h"
#include <algorithm> // for min(const _Tp __a,  const _Tp __b)
#import "Mandolin.h"

///////////////////////////
#pragma mark GLCircle
///////////////////////////


GLCircle::GLCircle( GLsizei count, CGPoint center, CGFloat size, 
                    Vector3D color, GLfloat line_width, bool filled ) {
  this->count = count;
  this->center = center;
  this->size = size;
  this->color.x = color.x;
  this->color.y = color.y;
  this->color.z = color.z;
  this->alpha = 1.0;
  this->line_width = line_width;
  this->filled = filled;
  this->animateOn = true;
  this->pressed = false;
  
  // picker animation
  this->size_animating = size;
  
  // Time increment
  this->dt = TIMEINCREMENT;
  
  // Z rotation
  this->zrot = 0.0;
}

// Draw Ellipse
void GLCircle::draw() {
  
  // ge: disable
  glDisable( GL_TEXTURE_2D );
  glDisable( GL_BLEND );
  
  glPushMatrix();
  
  CGFloat current_size = size;
  
  // Set Line Width
  glLineWidth(line_width);
  
  // Translate to center
  glTranslatef(center.x, center.y, 0.0f);

  
  // Animations
  if (this->animateOn) {
    // size animation
    if (pressed) {
      current_size = doExpandAnimation();
    }
    else {
      current_size = doShrinkAnimation();
    }

    // Rotation
    doRotateAnimation();
  }
  
  GLfloat vertices[count*2];
  GLfloat squareColors[count*4];
  
  int k=0, j=0; // arrays index
  for (GLfloat i = 0; i < 360.0f; i+=(360.0f/count))
  {
    // vertices
    vertices[k++] = (cos(degreesToRadian(i))*current_size);
    //NSLog(@"vertices[%d]: %f", count-1, vertices[count-1]);
    vertices[k++] = (sin(degreesToRadian(i))*current_size);
    //NSLog(@"vertices[%d]: %f", count-1, vertices[count-1]);
    
    // colors
    squareColors[j++] = fabs(cos(degreesToRadian(i))) * color.x;  // red
    squareColors[j++] = fabs(cos(degreesToRadian(i))) * color.y;  // green
    squareColors[j++] = fabs(cos(degreesToRadian(i))) * color.z;  // blue
    squareColors[j++] = alpha;                                    // alpha
    //NSLog(@"colors: [%d, %d, %d, %d]", squareColors[count-4], squareColors[count-3], squareColors[count-2], squareColors[count-1]);
  }
  
  glVertexPointer (2, GL_FLOAT , 0, vertices);
  glEnableClientState(GL_VERTEX_ARRAY);
  glColorPointer(4, GL_FLOAT, 0, squareColors);
  glEnableClientState(GL_COLOR_ARRAY);
  
  glDrawArrays ((filled) ? GL_TRIANGLE_FAN : GL_LINE_LOOP, 0, count);
  
  glPopMatrix();
}


// Perform the Expand animation
CGFloat GLCircle::doExpandAnimation() {
  
  CGFloat current_size;
  
  // increase
  if (size_animating < CIRCLEMAXSIZE) {
    size_animating += CIRCLEANIMATIONTIME;
  }
  
  current_size = size_animating;
  
  return current_size;
}

// Do the Shrinking animation
CGFloat GLCircle::doShrinkAnimation() {
  
  CGFloat current_size;
  
  if (size_animating > size) {
    //shrink
    size_animating -= CIRCLEANIMATIONTIME;
    current_size = size_animating;
  }
  else {
    current_size = size;
  }
  
  return current_size;
}

// Perform Rotation
void GLCircle::doRotateAnimation() {
  
  // Z Rotating
  glRotatef(zrot, 0.0, 0.0, 1.0f);
  
  // Increase zrot
  zrot += 1.5;
  
  // wrap zrot
  if (zrot >= 360) {
    zrot = 0.0;
  }
}

// Is the Circle Pressed?
bool GLCircle::isPressed() {
  return pressed;
}

// Set whether the Circle is Pressed or Not
void GLCircle::setPressed(bool p) {
  pressed = p;
}


///////////////////////////
#pragma mark GLExplosion
///////////////////////////


GLExplosion::GLExplosion( GLsizei count, CGPoint center, CGFloat size, 
                   GLubyte *color, GLfloat line_width, bool filled ) {
  this->count = count;
  this->center = center;
  this->size = size;
  this->color.x = color[0];
  this->color.y = color[1];
  this->color.z = color[2];
  this->alpha = 1.0;
  this->line_width = line_width;
  this->filled = filled;
  this->animateOn = true;
  
  // picker animation
  this->animation_time = EXPLOSIONANIMATION;
  this->size_animating = size;
    
  // Z rotation
  this->zrot = 0.0;
}

// Draw Ellipse
void GLExplosion::draw() {
  
  // ge: disable
  glDisable( GL_TEXTURE_2D );
  glEnable( GL_BLEND );
  
  glPushMatrix();
  
  CGFloat current_size = size;
  
  // Set Line Width
  glLineWidth(line_width);
  
  // Translate to center
  glTranslatef(center.x, center.y, 0.0f);
  
  
  // Animations
  if (this->animateOn) {
    // size animation
    if (animation_time > 0) {
      current_size = doSizeAnimation();
    }
    // Rotation
    doRotateAnimation();
  }
  
  GLfloat vertices[count*2];
  GLubyte squareColors[count*4];
  
  int k=0, j=0; // arrays index
  for (GLfloat i = 0; i < 360.0f; i+=(360.0f/count))
  {
    // vertices
    vertices[k++] = (cos(degreesToRadian(i))*current_size);
    //NSLog(@"vertices[%d]: %f", count-1, vertices[count-1]);
    vertices[k++] = (sin(degreesToRadian(i))*current_size);
    //NSLog(@"vertices[%d]: %f", count-1, vertices[count-1]);
    
    // colors
    squareColors[j++] = fabs(cos(degreesToRadian(i))) * color.x;  // red
    squareColors[j++] = fabs(cos(degreesToRadian(i))) * color.y;  // green
    squareColors[j++] = fabs(cos(degreesToRadian(i))) * color.z;  // blue
    squareColors[j++] = alpha * 255;                               // alpha
    //NSLog(@"colors: [%d, %d, %d, %d]", squareColors[count-4], squareColors[count-3], squareColors[count-2], squareColors[count-1]);
  }
  
  glVertexPointer (2, GL_FLOAT , 0, vertices);
  glEnableClientState(GL_VERTEX_ARRAY);
  glColorPointer(4, GL_UNSIGNED_BYTE, 0, squareColors);
  glEnableClientState(GL_COLOR_ARRAY);
  
  glDrawArrays ((filled) ? GL_TRIANGLE_FAN : GL_LINE_LOOP, 0, count);
  
  glPopMatrix();
}


// Perform the Size animation
CGFloat GLExplosion::doSizeAnimation() {
  
  CGFloat current_size;
  
  if (animation_time > 0) {
    // increase
    size_animating += 5.2;
    alpha -= 0.07;
    current_size = size_animating;
    animation_time--;
  }
  else {
    current_size = size;
  }
  
  return current_size;
}

// Perform Rotation
void GLExplosion::doRotateAnimation() {
  
  // Z Rotating
  glRotatef(zrot, 0.0, 0.0, 1.0f);
  
  // Increase zrot
  zrot += 5.5;
  
  // wrap zrot
  if (zrot >= 360) {
    zrot = 0.0;
  }
}


///////////////////////////
#pragma mark GLHole
///////////////////////////

GLHole::GLHole( CGPoint center, CGFloat size, Vector3D color, AudioData * data ) {

  this->center = center;
  this->size = size;
  this->color.x = color[0];
  this->color.y = color[1];
  this->color.z = color[2];
  this->alpha = 1.0;
  this->data = data;
  
  // picker animation
  this->size_animating = size;
  
  // Time increment
  this->dt = TIMEINCREMENT;
  
  // Z rotation
  this->zrot = 0.0;
}

// Draw Ellipse
void GLHole::draw() {
  
  // ge: enable
  glEnable( GL_TEXTURE_2D );
  glEnable( GL_BLEND );
  
  glPushMatrix();
  
  // Set Line Width
  glLineWidth(line_width);
  
  static const GLfloat squareVertices[] = {
    -0.5f,  -0.5f,
    0.5f,  -0.5f,
    -0.5f,   0.5f,
    0.5f,   0.5f,
  };
  
  GLfloat squareColors[] = {
    color.x,  color.y,  color.z,  alpha,
    color.x,  color.y,  color.z,  alpha,
    color.x,  color.y,  color.z,  alpha,
    color.x,  color.y,  color.z,  alpha,
  };
  
  static const GLfloat normals[] = {
    0, 0, 1,
    0, 0, 1,
    0, 0, 1,
    0, 0, 1
  };
  
  static const GLfloat texCoords[] = {
    0, 1,
    1, 1,
    0, 0,
    1, 0
  };
  
  // Translate to center
  glTranslatef(center.x, center.y, 0.0f);
  // Scale
  glScalef( size*2, size*2, 1 );
  
  // vertex
  glVertexPointer( 2, GL_FLOAT, 0, squareVertices );
  glEnableClientState(GL_VERTEX_ARRAY );
  
  // color
  //NSLog(@"r: %f, g: %f, %b: %f, alpha: %f", color.x, color.y, color.z, alpha);
  glColorPointer(4, GL_FLOAT, 0, squareColors);
  glEnableClientState(GL_COLOR_ARRAY);
  
  // normal
  glNormalPointer( GL_FLOAT, 0, normals );
  glEnableClientState( GL_NORMAL_ARRAY );
  
  // texture coordinate
  glTexCoordPointer( 2, GL_FLOAT, 0, texCoords );
  glEnableClientState( GL_TEXTURE_COORD_ARRAY );
  
  // triangle strip
  glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
  
  glPopMatrix();
}