/*
 *  GLUtils.mm
 *  hw2
 *
 *  Created by Uri Nieto on 1/24/10.
 *  Copyright 2010 Stanford University. All rights reserved.
 *
 */

#include "GLUtils.h"

// Degrees to Radian
GLfloat degreesToRadian( CGFloat degrees ) {
  
  return degrees * M_PI / 180.0;
}

// Compute the distance between the given points
CGFloat distanceBetweenPoints(CGPoint p1, CGPoint p2) {
  CGFloat x = fabs(p1.x - p2.x);
  CGFloat y = fabs(p1.y - p2.y);
  
  CGFloat d = norm(CGPointMake(x, y));
  
  return d;
}

// Compute the distance between the given vectors
CGFloat distanceBetweenVectors(Vector3D v1, Vector3D v2) {
  CGFloat x = fabs(v1.x - v2.x);
  CGFloat y = fabs(v1.y - v2.y);
  CGFloat z = fabs(v1.z - v2.z);
  
  Vector3D v;
  v.x =x; v.y = y; v.z = z;
  
  CGFloat d = v.magnitude();
  
  return d;
}

// Compute the middle point between two points
CGPoint middlePoint(CGPoint p1, CGPoint p2) {
  CGPoint middle;
  
  middle.x = (p1.x + p2.x) / 2.0;
  middle.y = (p1.y + p2.y) / 2.0;
  
  return middle;
}

// Computes the slope between two points relative to the X axis
CGFloat slopeBetweenPoints(CGPoint p1, CGPoint p2) {
  return (p1.y - p2.y) / (p1.x - p2.x);
}

// Computes the angle relative to the X axis (in rads) given the slope
CGFloat computeAngle(CGFloat slope) {
  return atan(slope);
}

// Computes the angle relative to the X axis (in rads) given y and x
CGFloat computeAngle(CGFloat y, CGFloat x) {
  return atan2(y, x);
}

// Random float between two floats
GLfloat rand2f( float a, float b )
{
  GLfloat diff = b - a;
  return a + ((GLfloat)rand() / RAND_MAX)*diff;
}

// Dot PRoduct between two points
CGFloat dotProduct(CGPoint p1, CGPoint p2) {
  return p1.x * p2.x + p1.y * p2.y;
}

// Return the norm of a 2D vector
CGFloat norm(CGPoint v) {
  return sqrt( v.x * v.x + v.y * v.y );
}

// Return the unit 2D vector
CGPoint unit(CGPoint v) {
  CGFloat n = norm(v);
  return CGPointMake(v.x / (CGFloat)n, v.y / (CGFloat)n);
}